part of '../mqtt.dart';

class MqttMessageSubscribe extends MqttMessage {
  int _messageID = 0;
  final List<String> _topics = [];
  final List<MqttQos> _qoss = [];

  List<String> get topics => _topics;

  void withMessageID(int val) {
    _messageID = val;
  }

  void add(String topic, MqttQos qos) {
    _topics.add(topic);
    _qoss.add(qos);
  }

  MqttMessageSubscribe.withTopic(int val, String topic, MqttQos qos) {
    fixedHead = MqttFixedHead().asType(MqttMessageType.subscribe);
    fixedHead.qos = MqttQos.qos1;
    withMessageID(val);
    add(topic, qos);
  }

  /// Initializes a new instance of the MqttConnectAckMessage class.
  MqttMessageSubscribe.fromByteBuffer(
      MqttFixedHead header, MqttBuffer messageStream) {
    fixedHead = header;
  }

  @override
  void writeTo(MqttBuffer messageStream) {
    fixedHead.messageType = MqttMessageType.subscribe;
    MqttBuffer _variableHeader = MqttBuffer();
    _variableHeader.writeInteger(_messageID);
    for (var i = 0; i < _topics.length; i++) {
      _variableHeader.writeUtf8String(_topics[i]);
      _variableHeader.writeBits(_qoss[i].index);
    }
    fixedHead.remainingLength = _variableHeader.length;
    messageStream.addAll(fixedHead.headerBytes());
    messageStream.addAll(_variableHeader.bytes);
  }

  @override
  String toString() =>
      fixedHead.toString() +
      "id: " +
      fixedHead.green(_messageID.toString().padRight(6)) +
      'topics: ' +
      fixedHead.green(fixedHead.yellow(_topics.join(",")));
}
